#!/opt/puppet/bin/ruby

require 'trollop'
require 'json'
require 'winrm'

@opts = Trollop::options do
  opt :server, 'scvmm server address', :type => :string
  opt :port, 'scvmm server port', :default => 5985
  opt :username, 'scvmm server username', :type => :string
  opt :password, 'scvmm server password', :type => :string
end

def winrm
  endpoint = "http://#{@opts[:server]}:#{@opts[:port]}/wsman"
  WinRM::WinRMWebService.new(
    endpoint, :plaintext,
    :user => @opts[:username],
    :pass => @opts[:password],
    :disable_sspi => true
  )
end

def facts
  import = 'Import-Module VirtualMachineManager;'

  data = Hash.new

  {
    :hostgroup => import + 'Get-SCVMHostGroup -VMMServer localhost | Select Path | ConvertTo-Json',
    :host => import + 'Get-SCVMHost -VMMServer localhost | Select Name,{$_.HostCluster.ClusterName},{$_.VMHostGroup.Path} | ConvertTo-Json',
    :vm => import + 'Get-SCVirtualMachine -VMMServer localhost | Select Name,MarkedAsTemplate,HostGroupPath,{$_.OperatingSystem.Name},HostType | ConvertTo-Json',
    :template => import + 'Get-Template -VMMServer localhost | Select Name,{$_.OperatingSystem.Name} | ConvertTo-Json',
  }.each do |res_type, command|
    result = ''
    winrm.powershell(command) do |stdout, stderr|
      result += stdout.strip unless stdout.nil?
    end
    begin
      data[res_type] = JSON.parse(result)
    rescue JSON::ParserError
      data[res_type] = []
    end
    data[res_type] = [data[res_type]] unless data[res_type].is_a? Array
  end
  data
end

begin
  results = facts.to_json
  exit 0
rescue WinRM, HTTPClient => e
  puts e.message
  exit 1
ensure
  results ||= {}
  scvmm_cache = '/opt/Dell/ASM/cache'
  Dir.mkdir(scvmm_cache) unless Dir.exists? scvmm_cache
  file_path = File.join(scvmm_cache, "#{@opts[:server]}.json")
  File.write(file_path, results) unless results.empty?
end
