unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

Puppet::Type.type(:iscsi_target).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    command  = exist_cmd + "| where { $_.NodeAddress -match #{quote(resource[:name])}} | format-list | Out-String -width 4096"
    result = winrm_ps(command)
    @nodeaddress = result.scan(/^NodeAddress\s+:\s+(.*)$/).collect{|v| v.first.chomp} || []
    ! (result.include? "IsConnected    : False")
  end

  def connect
    raise PuppetError, 'Unable to find suitable nodeaddress' unless @nodeaddress
    commands = @nodeaddress.collect do |address|
      command = ps_cmd( 'Connect-IscsiTarget',
        {
          :NodeAddress => quote(address),
          :IsPersistent => resource[:is_persistent],
          :IsMultipathEnabled => resource[:is_multipath_enabled],
          :TargetPortalAddress => resource[:target_portal_address],
          :TargetPortalPortNumber => resource[:target_portal_port_number],
          :AuthenticationType => resource[:authentication_type],
          :InitiatorInstanceName => resource[:initiator_instance_name],
        }
      )
      winrm_ps(command)
    end
  end

  def disconnect
    @nodeaddress.each do |address|
      command = ps_cmd('Disconnect-IscsiTarget',
        {
          :NodeAddress => quote(address),
          :Confirm => '$false',
        }
      )
      winrm_ps(command)
    end
  end

  def exist_cmd
    command = ps_cmd( 'Get-IscsiTarget', {}, :terminate => false)
  end
end
