unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

Puppet::Type.type(:iscsi_target_portal).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    # Do not truncate output:
    command  = exist_cmd + '| Out-String -width 4096'
    result = winrm_ps(command)
    result.include? resource[:name]
  rescue RuntimeError
    false
  end

  def create
    winrm_ps(create_cmd)
  end

  def destroy
    command = ps_cmd('Remove-IscsiTargetPortal',
      {
        :InitiatorInstanceName => quote(resource[:name]),
        :TargetPortalAddress => resource[:target_portal_address],
        :Confirm => '$false',
      }
    )

    winrm_ps(command)
  end

  def exist_cmd
    command = ps_cmd( 'Get-IscsiTargetPortal',
      {
        :InitiatorInstanceName => quote(resource[:name]),
      },
      :terminate => false,
    )
  end

  def create_cmd
    ps_cmd( 'New-IscsiTargetPortal',
      {
        :InitiatorInstanceName => quote(resource[:name]),
        :TargetPortalAddress => resource[:target_portal_address],
        :TargetPortalPortNumber => resource[:target_portal_port_number],
        :AuthenticationType => resource[:authentication_type],
        :ChapUsername => resource[:chap_username],
        :ChapSecret => resource[:chap_secret],
      }
    )
  end
end
